// A basic OpenGL program displaying streamlines

#include <windows.h>
#include <gl/gl.h>
#include <gl/glu.h>
#include <gl/glut.h>
#include <iostream>
using namespace std;
#include "Trackball.h"
#include "Geometry.h"

const int windowWidth=400;
const int windowHeight=400;

const float deltaT=0.1;
const int numVertices=100;
CVec3df solutionVertices[numVertices+1];
CVec3df eulerVertices[numVertices+1];
CVec3df midpointVertices[numVertices+1];

CTrackball trackball;

// gives the vector field at point p
inline CVec3df vectorField(CVec3df& p)
{
	return CVec3df(2*p[Y], -2*p[X], 0.1);
}

void handleMouseMotion(int x, int y)
{
	trackball.tbMotion(x, y);
}

void handleMouseClick(int button, int state, int x, int y)
{
	trackball.tbMouse(button, state, x, y);
}

void handleKeyboardEvent(unsigned char key, int x, int y)
{
	trackball.tbKeyboard(key);
}

void display(void)
{
	glClear(GL_COLOR_BUFFER_BIT); 	// clear all pixels in frame buffer
	glMatrixMode( GL_MODELVIEW );	// Set the view matrix ...
	glLoadIdentity();				// ... to identity.
	trackball.tbMatrix();

	// draw the ground mesh
	glColor3f (1.0, 0.0, 0.0);
	glBegin(GL_LINES);
	float c;
	for(int i=0;i<11;i++){
		c=-2.0f+4.0f*((float) i)/10.0f;
		glVertex3f(c,-2,0);
		glVertex3f(c,2,0);
		glVertex3f(-2,c,0);
		glVertex3f(2,c,0);
	}
	glEnd();

	// draw the correct solution
	glColor3f (1.0, 1.0, 0.0);
	glBegin(GL_LINE_STRIP);
	for(int i=0;i<numVertices;i++)
		glVertex3fv(solutionVertices[i].getArray());
	glEnd();

	// draw the numerical solution computed with the Euler method
	// PLEASE INSERT MISSING CODE BELOW

	// draw the numerical solution computed with the Mid-point method
	// PLEASE INSERT MISSING CODE BELOW

	glFlush ();
}

void init(void)
{
	// select clearing color (for glClear)
	glClearColor (0.5, 0.5, 0.5, 0.0);	// RGB-value for gray
	// initialize view (simple orthographic projection)
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(15.0, (GLfloat) windowWidth/(GLfloat) windowHeight, 5.0, 55.0);
	gluLookAt(0,0,30, 0,0,0, 0,1,0);
	trackball.tbInit(GLUT_LEFT_BUTTON);

	// compute vertices for the exact solution
	for(int i=0;i<numVertices;i++)
	{
		double t=3.0*Pi*((double) i)/((double) numVertices);
		solutionVertices[i].setVector(sin(2*t), cos(2*t), 0.1*t);
	}

	// compute vertices with the Euler method
	// PLEASE INSERT MISSING CODE BELOW

	// compute vertices with the Mid-point method
	// PLEASE INSERT MISSING CODE BELOW
}

void reshape(int width, int height ) {
	// Called at start, and whenever user resizes component
	int size = min(width, height);
	glViewport(0, 0, size, size);  // Largest possible square
	trackball.tbReshape(width, height);
}


// create a single buffered colour window
int main(int argc, char** argv)
{
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB);
	glutInitWindowSize(windowWidth, windowHeight);
	glutInitWindowPosition(100, 100);
	glutCreateWindow("Streamline Computation Example");
	init ();						// initialise view
	glutMouseFunc(handleMouseClick);		// Set function to handle mouse clicks
	glutMotionFunc(handleMouseMotion);		// Set function to handle mouse motion
	glutKeyboardFunc(handleKeyboardEvent);	// Set function to handle keyboard input
	glutDisplayFunc(display);		// Set function to draw scene
	glutReshapeFunc(reshape);		// Set function called if window gets resized
	glutMainLoop();
	return 0;
}
